"""
Module realizes base plugin class
"""
from abc import ABC, abstractmethod
from typing import TypeVar

from .plugin_meta import PluginMeta

T_LUNA_APPLICATION = TypeVar("T_LUNA_APPLICATION")


class BasePlugin(ABC, metaclass=PluginMeta):
    """
    Base abstract class.
    Attributes:
        app: running application
    """

    def __init__(self, app: T_LUNA_APPLICATION):
        self.app = app

    @property
    def pluginName(self) -> str:
        """
        Return plugin name

        Returns:
            class name
        """
        return type(self).__name__

    @abstractmethod
    async def initialize(self):
        """
        Initialize plugin.

        Open connections, etc
        """

    @abstractmethod
    async def close(self):
        """
        Stop plugin.

        Close all open connections, etc
        """


class BaseEventPlugin(BasePlugin):
    """
    Base class for plugin which is triggered on an event.
    """

    @property
    @abstractmethod
    def eventName(self) -> str:
        """
        Event name for handling

        Returns:
            event name
        """

    @abstractmethod
    async def handleEvent(self, *args, **kwargs):
        """
        Handle event

        Args:
            *args: positional arg for event handler function
            **kwargs: named arg for event handler function
        """


class BaseBackgroundPlugin(BasePlugin):
    """
    Base plugin for background process
    """

    @abstractmethod
    async def start(self):
        """
        Start background process.

        .. warning::
            The function suppose that the process is handle in this coroutine. The coroutine must start
            the process only without awaiting a end of the process
        """
