"""
Module realizes base matcher plugin and interface classes
"""
import json
import os
from abc import abstractmethod
from typing import Any, Optional

from luna3.configurator.configurator import ConfiguratorApi
from stringcase import snakecase

from ..base.plugins_meta.base_plugins import BasePlugin
from ..base.plugins_meta.plugin_meta import PluginMeta

# allowed plugin meta options
from ..utils.configs import parseConfiguratorAddress
from ..utils.exceptions import LunaPluginException
from .base_struct import IMatcher

ALLOWED_PLUGIN_OPTIONS = ("configurator", "file", None)


class MetaOptions(object):
    """
    Meta options for matcher plugins.
    """

    def __init__(self, meta: "Meta"):
        """
        Init class options.

        Args:
            meta: stored meta options.
        """
        self.configSource = getattr(meta, "configSource", None)
        self.configFile = getattr(meta, "configFile", None)
        self.tag = getattr(meta, "tag", None)
        if self.configSource not in ALLOWED_PLUGIN_OPTIONS:
            raise ValueError(f"The `configSource` option must be one of: {ALLOWED_PLUGIN_OPTIONS}")
        if self.tag is not None:
            if not isinstance(self.tag, str):
                raise ValueError("The `tag` option must be string")
        if self.configSource == "file":
            if not self.configFile:
                raise ValueError("The `configFile` option is required for the 'file' source")
            if not isinstance(self.configFile, str):
                raise ValueError("The `configFile` option must be string")


class MatcherPluginMeta(PluginMeta):
    """
    Metaclass for matcher plugins.
    """

    def __new__(mcs, name, bases, attrs):
        """
        Metaclass new method

        Args:
            name: class name
            bases: bases classes
            attrs: class attributes

        Returns:
            new class
        """
        meta = attrs.pop("Meta", None)
        obj = super().__new__(mcs, name, bases, attrs)
        if meta:
            obj.options = MetaOptions(meta=meta)
        return obj


class BaseMatcherPlugin(BasePlugin, metaclass=MatcherPluginMeta):
    """Base matcher plugin."""

    # config for matcher
    matcherConfig: Optional[dict[str, Any]] = None

    class Meta:
        """Options object for a Schema.

        Example usage: ::

            class Meta:
                configSource = "configurator"

            or

            class Meta:
                configSource = "file"
                configFile = "../source/configs/best_plugin_config.json"

        Available options:

            - ``configSource``: Source type for pulling matcher settings, must be one of ["configurator", "file", None]
            - ``configFile``: Path to custom json file with matcher configuration, if source type is file.
            - ``tag``: Tag for pulling a tagged setting from the configurator, if `configSource` is "configurator"
        """

    async def initialize(self) -> None:
        """Initialize plugin and load config for matcher."""
        await self.loadMatcherConfig()

    @abstractmethod
    def getMatcher(self) -> IMatcher:
        """
        Get matcher class.

        Returns:
            class `~IMatcher`
        """

    async def loadMatcherConfig(self) -> None:
        """
        Load matcher config.
        """
        if self.options.configSource is None:
            # skip loading matcher config
            return
        elif self.options.configSource == "configurator":
            if not self.app.ctx.serviceConfig._configuratorAddress:
                raise LunaPluginException("Configurator address is not defined in the service")

            configurator = ConfiguratorApi(*parseConfiguratorAddress(self.app.ctx.serviceConfig._configuratorAddress))
            settingName = f"LUNA_{snakecase(self.pluginName).upper()}"

            response = await configurator.getSettings(settingName=settingName, tags=self.options.tag, asyncRequest=True)
            if not response.success:
                raise LunaPluginException(f"Error while pulling settings from luna-configurator: {response.json}")
            elif not (settings := response.json["settings"]):
                raise LunaPluginException(f"Setting '{settingName}' for plugin {self.pluginName} not found")
            self.matcherConfig = settings[0]["value"]
        elif self.options.configSource == "file":
            if not os.path.isfile(self.options.configFile):
                raise LunaPluginException(f"Configuration file '{self.options.configFile}' not found")
            with open(self.options.configFile) as _conf:
                self.matcherConfig = json.loads(_conf.read())
        else:
            raise RuntimeError(f"Unsupported config source: {self.options.configSource}")
